#!/bin/bash
#

# build_mongodb.sh <product_directory> <e6|e7|e9> <debug|opt|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e5|e6|e7> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
    arch="osx"
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    mkdir -p $pkgdir
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
    
    if [[ "${OS1}" == Linux ]]; then
      arch="linux"
      if [[ "${OS}" == slf6-x86_64 ]]; then
        osname="rhel62-"
      elif [[ "${OS}" == slf7-x86_64 ]]; then
        osname="rhel70-"
      elif [[ "${OS}" == u14-x86_64 ]]; then
        osname="ubuntu1410-clang-"
      fi
    fi
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=mongodb
cppver=1.0.6
cppname=legacy
cver=1.2.0
origpkgver=v3_0_7
pkgver=${origpkgver}
ssibuildshims_version=v0_08_06
cppsourcetarname=${cppname}-${cppver}
csourcetarname=mongo-c-driver-${cver}

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

tarver=`echo $pkgver|sed -e 's/_/\./g'|sed -e 's/^v//g'`
pkgtarname=mongodb-${arch}-x86_64-${osname}${tarver}


if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi


if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS}
setup cmake v3_3_2
#ups active

# Common configure args

common_config_args="--prefix=${pkgdir} --enable-shared"

if [ "${extraqual}" = "debug" ]; then
    cmake_build_type="Debug"
    extra_config_args="--enable-debug --enable-tracing"
elif [ "${extraqual}" = "prof" ]; then
    cmake_build_type="RelWithDebInfo"
    extra_config_args="--enable-debug-symbols=yes"
else
    cmake_build_type="Release"
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x 

cd "${sourcedir}"
tar xf ${tardir}/${csourcetarname}.tar.gz || die "Unable to untar source 2."
tar xf ${tardir}/${pkgtarname}.tgz || die "Unable to untar source."

# Get the known-working version of the source. UPDATE CHECKOUT COMMIT AS NECESSARY!
git clone -b master https://github.com/mongodb/mongo-cxx-driver
cd mongo-cxx-driver
git checkout 3ea574271f152981b14344efccc4e8ae015b6b62
cd ..

# Install Binaries
mv ${pkgtarname}/bin ${pkgdir}

# Configure & Make C Driver
cd ${csourcetarname}
${sourcedir}/${csourcetarname}/configure ${common_config_args} ${extra_config_args} || die "ERROR: configure failed."
make -j ${ncore} || die "ERROR: c-driver make failed."
make install || die "ERROR: c-driver make install failed."
cd ..

# Configure & Make C++ Driver
cd mongo-cxx-driver/build
PKG_CONFIG_PATH=${pkgdir}/lib/pkgconfig cmake -DCMAKE_BUILD_TYPE=${cmake_build_type} -DCMAKE_INSTALL_PREFIX=${pkgdir} ..
make -j ${ncore} || die "ERROR: cxx-driver make failed."
make install || die "ERROR: cxx-driver make install failed."
set +x

# Put includes in correct location
mkdir ${pkgdir}/../include
mv ${pkgdir}/include/bsoncxx/v*/bsoncxx ${pkgdir}/../include
mv ${pkgdir}/include/mongocxx/v*/mongocxx ${pkgdir}/../include
rm -rf ${pkgdir}/include/bsoncxx ${pkgdir}/include/mongocxx
mv ${pkgdir}/include/* ${pkgdir}/../include
rm -rf ${pkgdir}/include

# Install CMake Config Files
cp -a ${patchdir}/mongodb ${pkgdir}/lib

# Really don't want the build files sticking around
rm -rf ${sourcedir}

# Clean up.
cd "${pkgdir}"

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "mongodb is installed at ${MONGODB_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
